@@ -135,7 +135,7 @@ Supporting OS subroutines required: <<close>>, <<fstat>>, <<isatty>>,
 
 #if defined(LIBC_SCCS) && !defined(lint)
 /*static char *sccsid = "from: @(#)vfprintf.c	5.50 (Berkeley) 12/16/92";*/
-static char *rcsid = "$Id: vfprintf.c,v 1.12 2006/09/26 21:22:19 jjohnstn Exp $";
+static char *rcsid = "$Id: vfprintf.c,v 1.14 2008/02/12 18:07:55 jjohnstn Exp $";
 #endif /* LIBC_SCCS and not lint */
 
 /*
@@ -207,7 +207,8 @@ typedef union
  * then reset it so that it can be reused.
  */
 static int
-__sprint(fp, uio)
+__sprint_r(rptr, fp, uio)
+	struct _reent *rptr;
 	FILE *fp;
 	register struct __suio *uio;
 {
@@ -217,7 +218,7 @@ __sprint(fp, uio)
 		uio->uio_iovcnt = 0;
 		return (0);
 	}
-	err = __sfvwrite(fp, uio);
+	err = __sfvwrite_r(rptr, fp, uio);
 	uio->uio_resid = 0;
 	uio->uio_iovcnt = 0;
 	return (err);
@@ -229,7 +230,8 @@ __sprint(fp, uio)
  * worries about ungetc buffers and so forth.
  */
 static int
-__sbprintf(fp, fmt, ap)
+__sbprintf_r(rptr, fp, fmt, ap)
+	struct _reent *rptr;
 	register FILE *fp;
 	const char *fmt;
 	va_list ap;
@@ -250,8 +252,8 @@ __sbprintf(fp, fmt, ap)
 	fake._lbfsize = 0;	/* not actually used, but Just In Case */
 
 	/* do the work, then copy any error status */
-	ret = VFPRINTF(&fake, fmt, ap);
-	if (ret >= 0 && fflush(&fake))
+	ret = _VFPRINTF_R(rptr, &fake, fmt, ap);
+	if (ret >= 0 && _fflush_r(rptr, &fake))
 		ret = EOF;
 	if (fake._flags & __SERR)
 		fp->_flags |= __SERR;
@@ -366,13 +368,12 @@ _DEFUN (_VFPRINTF_R, (data, fp, fmt0, ap),
 #ifndef _NO_LONGLONG
 #define	quad_t	  long long
 #define	u_quad_t  unsigned long long
+#else
+#define quad_t    long
+#define u_quad_t  u_long
 #endif
 
-#ifndef _NO_LONGLONG
 	u_quad_t _uquad;	/* integer arguments %[diouxX] */
-#else
-	u_long _uquad;
-#endif
 	enum { OCT, DEC, HEX } base;/* base for [diouxX] conversion */
 	int dprec;		/* a copy of prec if [diouxX], 0 otherwise */
 	int realsz;		/* field size expanded by dprec */
@@ -410,7 +411,7 @@ _DEFUN (_VFPRINTF_R, (data, fp, fmt0, ap),
 	uio.uio_resid += (len); \
 	iovp++; \
 	if (++uio.uio_iovcnt >= NIOV) { \
-		if (__sprint(fp, &uio)) \
+		if (__sprint_r(data, fp, &uio)) \
 			goto error; \
 		iovp = iov; \
 	} \
@@ -425,7 +426,7 @@ _DEFUN (_VFPRINTF_R, (data, fp, fmt0, ap),
 	} \
 }
 #define	FLUSH() { \
-	if (uio.uio_resid && __sprint(fp, &uio)) \
+	if (uio.uio_resid && __sprint_r(data, fp, &uio)) \
 		goto error; \
 	uio.uio_iovcnt = 0; \
 	iovp = iov; \
@@ -520,13 +521,15 @@ _DEFUN (_VFPRINTF_R, (data, fp, fmt0, ap),
         memset (&state, '\0', sizeof (state));
 
 	/* sorry, fprintf(read_only_file, "") returns EOF, not 0 */
-	if (cantwrite(fp))
+	if (cantwrite (data, fp)) {
+		_funlockfile (fp);	
 		return (EOF);
+	}
 
 	/* optimise fprintf(stderr) (and other unbuffered Unix files) */
 	if ((fp->_flags & (__SNBF|__SWR|__SRW)) == (__SNBF|__SWR) &&
 	    fp->_file >= 0)
-		return (__sbprintf(fp, fmt0, ap));
+		return (__sbprintf_r(data, fp, fmt0, ap));
 
 	fmt = (char *)fmt0;
 	uio.uio_iov = iovp = iov;
@@ -537,9 +540,10 @@ _DEFUN (_VFPRINTF_R, (data, fp, fmt0, ap),
 	/*
 	 * Scan the format for conversions (`%' character).
 	 */
+
 	for (;;) {
 	        cp = fmt;
-	        while ((n = _mbtowc_r(_REENT, &wc, fmt, MB_CUR_MAX, &state)) > 0) {
+	        while ((n = _mbtowc_r(data, &wc, fmt, MB_CUR_MAX, &state)) > 0) {
 			fmt += n;
 			if (wc == '%') {
 				fmt--;
@@ -750,14 +754,9 @@ reswitch:	switch (ch) {
 			  }
 #endif /* __ALTIVEC__ */
 			_uquad = SARG();
-#ifndef _NO_LONGLONG
 			if ((quad_t)_uquad < 0)
-#else
-			if ((long) _uquad < 0)
-#endif
 			{
-
-				_uquad = -_uquad;
+				_uquad = -(quad_t)_uquad;
 				old_sign = sign;
 				sign = '-';
 			}
@@ -903,11 +902,11 @@ reswitch:	switch (ch) {
 			    _uquad = -(quad_t)_uquad;
 			  }
 			if (flags & SHORTINT)
-			  _uquad <<= 49;
+			  _uquad <<= (sizeof(quad_t) - sizeof(short)) * 8 + 1;
 			else if (flags & LONGINT)
 			  _uquad <<= 1;
 			else
-			  _uquad <<= 33;
+			  _uquad <<= (sizeof(quad_t) - sizeof(long)) * 8 + 1;
 
 			if (_uquad == 0 && sign)
 			  {
@@ -923,15 +922,19 @@ reswitch:	switch (ch) {
 		        flags |= FIXEDPOINT;
 		        _uquad = UFPARG();
 			if (flags & SHORTINT)
-			  _uquad <<= 48;
+			  _uquad <<= (sizeof(quad_t) - sizeof(short)) * 8;
 			else if (!(flags & LONGINT))
-			  _uquad <<= 32;
+			  _uquad <<= (sizeof(quad_t) - sizeof(long)) * 8;
 	
 fixed_nosign:
 			if (prec == -1)
 			  prec = DEFPREC;
 
+#ifndef _NO_LONGLONG
 			cp = cvt_ufix64 (data, _uquad, prec, &expt, &ndig);
+#else
+			cp = cvs_ufix32 (data, _uquad, prec, &expt, &ndig);
+#endif
 
 			/* act like %f of format "0.X" */
 			size = prec + 2;
