@@ -20,11 +20,8 @@ extern void (*__preinit_array_start []) (void) __attribute__((weak));
 extern void (*__preinit_array_end []) (void) __attribute__((weak));
 extern void (*__init_array_start []) (void) __attribute__((weak));
 extern void (*__init_array_end []) (void) __attribute__((weak));
-extern void (*__fini_array_start []) (void) __attribute__((weak));
-extern void (*__fini_array_end []) (void) __attribute__((weak));
 
 extern void _init (void);
-extern void _fini (void);
 
 /* Iterate over all the init routines.  */
 void
@@ -43,18 +40,4 @@ __libc_init_array (void)
   for (i = 0; i < count; i++)
     __init_array_start[i] ();
 }
-
-/* Run all the cleanup routines.  */
-void
-__libc_fini_array (void)
-{
-  size_t count;
-  size_t i;
-  
-  count = __fini_array_end - __fini_array_start;
-  for (i = count; i > 0; i--)
-    __fini_array_start[i-1] ();
-
-  _fini ();
-}
 #endif
