@@ -19,12 +19,13 @@ INDEX
 ANSI_SYNOPSIS
 	#include <stdio.h>
 	#include <stdarg.h>
-	int dprintf(int <[fd]>, const char *<[format]>, ...);
-	int vdprintf(int <[fd]>, const char *<[format]>, va_list <[ap]>);
+	int dprintf(int <[fd]>, const char *restrict <[format]>, ...);
+	int vdprintf(int <[fd]>, const char *restrict <[format]>,
+			va_list <[ap]>);
 	int _dprintf_r(struct _reent *<[ptr]>, int <[fd]>,
-			const char *<[format]>, ...);
+			const char *restrict <[format]>, ...);
 	int _vdprintf_r(struct _reent *<[ptr]>, int <[fd]>,
-			const char *<[format]>, va_list <[ap]>);
+			const char *restrict <[format]>, va_list <[ap]>);
 
 DESCRIPTION
 <<dprintf>> and <<vdprintf>> allow printing a format, similarly to
@@ -55,7 +56,7 @@ int
 _DEFUN(_dprintf_r, (ptr, fd, format),
        struct _reent *ptr _AND
        int fd _AND
-       const char *format _DOTS)
+       const char *__restrict format _DOTS)
 {
 	va_list ap;
 	int n;
@@ -71,7 +72,7 @@ _DEFUN(_dprintf_r, (ptr, fd, format),
 int
 _DEFUN(dprintf, (fd, format),
        int fd _AND
-       const char *format _DOTS)
+       const char *__restrict format _DOTS)
 {
   va_list ap;
   int n;
