@@ -62,7 +62,11 @@ _DEFUN(std, (ptr, flags, file, data),
   ptr->_flags |= __SL64;
 #endif /* __LARGE64_FILES */
   ptr->_seek = __sseek;
+#ifdef _STDIO_CLOSE_PER_REENT_STD_STREAMS
   ptr->_close = __sclose;
+#else /* _STDIO_CLOSE_STD_STREAMS */
+  ptr->_close = NULL;
+#endif /* _STDIO_CLOSE_STD_STREAMS */
 #if !defined(__SINGLE_THREAD__) && !defined(_REENT_SMALL)
   __lock_init_recursive (ptr->_lock);
   /*
@@ -77,23 +81,27 @@ _DEFUN(std, (ptr, flags, file, data),
 #endif
 }
 
+struct glue_with_file {
+  struct _glue glue;
+  FILE file;
+};
+
 struct _glue *
 _DEFUN(__sfmoreglue, (d, n),
        struct _reent *d _AND
        register int n)
 {
-  struct _glue *g;
-  FILE *p;
+  struct glue_with_file *g;
 
-  g = (struct _glue *) _malloc_r (d, sizeof (*g) + n * sizeof (FILE));
+  g = (struct glue_with_file *)
+    _malloc_r (d, sizeof (*g) + (n - 1) * sizeof (FILE));
   if (g == NULL)
     return NULL;
-  p = (FILE *) (g + 1);
-  g->_next = NULL;
-  g->_niobs = n;
-  g->_iobs = p;
-  memset (p, 0, n * sizeof (FILE));
-  return g;
+  g->glue._next = NULL;
+  g->glue._niobs = n;
+  g->glue._iobs = &g->file;
+  memset (&g->file, 0, n * sizeof (FILE));
+  return &g->glue;
 }
 
 /*
