@@ -14,35 +14,42 @@
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
+/* doc in sprintf.c */
 
 #include <_ansi.h>
+#include <reent.h>
 #include <stdio.h>
-#ifdef _HAVE_STDC
 #include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
 
-#ifdef _HAVE_STDC
 int
-fprintf(FILE * fp, _CONST char *fmt,...)
-#else
-int
-fprintf(fp, fmt, va_alist)
-        FILE *fp;
-        char *fmt;
-        va_dcl
-#endif
+_DEFUN(_fprintf_r, (ptr, fp, fmt),
+       struct _reent *ptr _AND
+       FILE *__restrict fp _AND
+       const char *__restrict fmt _DOTS)
 {
   int ret;
   va_list ap;
 
-#ifdef _HAVE_STDC
   va_start (ap, fmt);
-#else
-  va_start (ap);
-#endif
-  ret = vfprintf (fp, fmt, ap);
+  ret = _vfprintf_r (ptr, fp, fmt, ap);
   va_end (ap);
   return ret;
 }
+
+#ifndef _REENT_ONLY
+
+int
+_DEFUN(fprintf, (fp, fmt),
+       FILE *__restrict fp _AND
+       const char *__restrict fmt _DOTS)
+{
+  int ret;
+  va_list ap;
+
+  va_start (ap, fmt);
+  ret = _vfprintf_r (_REENT, fp, fmt, ap);
+  va_end (ap);
+  return ret;
+}
+
+#endif /* ! _REENT_ONLY */
