@@ -79,9 +79,9 @@ _DEFUN(__getdelim, (bufptr, n, delim, fp),
       *n = DEFAULT_LINE_SIZE;
     }
 
-  CHECK_INIT (_REENT);
+  CHECK_INIT (_REENT, fp);
 
-  _flockfile (fp);
+  _newlib_flockfile_start (fp);
 
   numbytes = *n;
   ptr = buf;
@@ -129,7 +129,7 @@ _DEFUN(__getdelim, (bufptr, n, delim, fp),
         }
     }
 
-  _funlockfile (fp);
+  _newlib_flockfile_end (fp);
 
   /* if no input data, return failure */
   if (ptr == buf)
