@@ -29,7 +29,7 @@
 
 int
 #ifdef _HAVE_STDC
-scanf(_CONST char *fmt, ...)
+scanf(_CONST char *__restrict fmt, ...)
 #else
 scanf(fmt, va_alist)
       char *fmt;
@@ -38,14 +38,15 @@ scanf(fmt, va_alist)
 {
   int ret;
   va_list ap;
+  struct _reent *reent = _REENT;
 
-  _REENT_SMALL_CHECK_INIT (_REENT);
+  _REENT_SMALL_CHECK_INIT (reent);
 #ifdef _HAVE_STDC
   va_start (ap, fmt);
 #else
   va_start (ap);
 #endif
-  ret = __svfscanf_r (_REENT, _stdin_r (_REENT), fmt, ap);
+  ret = _vfscanf_r (reent, _stdin_r (reent), fmt, ap);
   va_end (ap);
   return ret;
 }
@@ -54,7 +55,7 @@ scanf(fmt, va_alist)
 
 int
 #ifdef _HAVE_STDC
-_scanf_r(struct _reent *ptr, _CONST char *fmt, ...)
+_scanf_r(struct _reent *ptr, _CONST char *__restrict fmt, ...)
 #else
 _scanf_r(ptr, fmt, va_alist)
          struct _reent *ptr;
@@ -71,7 +72,7 @@ _scanf_r(ptr, fmt, va_alist)
 #else
   va_start (ap);
 #endif
-  ret = __svfscanf_r (ptr, _stdin_r (ptr), fmt, ap);
+  ret = _vfscanf_r (ptr, _stdin_r (ptr), fmt, ap);
   va_end (ap);
   return (ret);
 }
