@@ -17,69 +17,75 @@
 
 /*
 FUNCTION
-        <<iprintf>>, <<fiprintf>>, <<asiprintf>>, <<siprintf>>, <<sniprintf>>---format output
+<<siprintf>>, <<fiprintf>>, <<iprintf>>, <<sniprintf>>, <<asiprintf>>, <<asniprintf>>---format output (integer only)
 
 INDEX
 	fiprintf
+INDEX
+	_fiprintf_r
 INDEX
 	iprintf
 INDEX
-	asiprintf
+	_iprintf_r
 INDEX
 	siprintf
+INDEX
+	_siprintf_r
 INDEX
 	sniprintf
+INDEX
+	_sniprintf_r
+INDEX
+	asiprintf
+INDEX
+	_asiprintf_r
+INDEX
+	asniprintf
+INDEX
+	_asniprintf_r
 
 ANSI_SYNOPSIS
         #include <stdio.h>
 
-        int iprintf(const char *<[format]> [, <[arg]>, ...]);
-        int fiprintf(FILE *<[fd]>, const char *<[format]> [, <[arg]>, ...]);
-        int siprintf(char *<[str]>, const char *<[format]> [, <[arg]>, ...]);
-        int asiprintf(char **<[strp]>, const char *<[format]> [, <[arg]>, ...]);
-        int sniprintf(char *<[str]>, size_t <[size]>, const char *<[format]>
-                      [, <[arg]>, ...]);
+        int iprintf(const char *<[format]>, ...);
+        int fiprintf(FILE *<[fd]>, const char *<[format]> , ...);
+        int siprintf(char *<[str]>, const char *<[format]>, ...);
+        int sniprintf(char *<[str]>, size_t <[size]>, const char *<[format]>, 
+			...);
+        int asiprintf(char **<[strp]>, const char *<[format]>, ...);
+        char *asniprintf(char *<[str]>, size_t *<[size]>, 
+			const char *<[format]>, ...);
 
-TRAD_SYNOPSIS
-	#include <stdio.h>
-
-	int iprintf(<[format]> [, <[arg]>, ...])
-	char *<[format]>;
-
-	int fiprintf(<[fd]>, <[format]> [, <[arg]>, ...]);
-	FILE *<[fd]>;
-	char *<[format]>;
-
-	int asiprintf(<[strp]>, <[format]> [, <[arg]>, ...]);
-	char **<[strp]>;
-	char *<[format]>;
-
-	int siprintf(<[str]>, <[format]> [, <[arg]>, ...]);
-	char *<[str]>;
-	char *<[format]>;
-
-	int sniprintf(<[str]>, size_t <[size]>, <[format]> [, <[arg]>, ...]);
-	char *<[str]>;
-        size_t <[size]>;
-	char *<[format]>;
+        int _iprintf_r(struct _reent *<[ptr]>, const char *<[format]>, ...);
+        int _fiprintf_r(struct _reent *<[ptr]>, FILE *<[fd]>,
+                        const char *<[format]>, ...);
+        int _siprintf_r(struct _reent *<[ptr]>, char *<[str]>,
+                        const char *<[format]>, ...);
+        int _sniprintf_r(struct _reent *<[ptr]>, char *<[str]>, size_t <[size]>,
+                         const char *<[format]>, ...);
+        int _asiprintf_r(struct _reent *<[ptr]>, char **<[strp]>,
+                         const char *<[format]>, ...);
+        char *_asniprintf_r(struct _reent *<[ptr]>, char *<[str]>,
+                            size_t *<[size]>, const char *<[format]>, ...);
 
 DESCRIPTION
         <<iprintf>>, <<fiprintf>>, <<siprintf>>, <<sniprintf>>,
-        <<asiprintf>>, are the same as <<printf>>, <<fprintf>>,
-	<<sprintf>>, <<snprintf>>, and <<asprintf>>, respectively,
-	only that they restrict usage to non-floating-point format
-	specifiers.
+        <<asiprintf>>, and <<asniprintf>> are the same as <<printf>>,
+        <<fprintf>>, <<sprintf>>, <<snprintf>>, <<asprintf>>, and
+        <<asnprintf>>, respectively, except that they restrict usage
+        to non-floating-point format specifiers.
+
+        <<_iprintf_r>>, <<_fiprintf_r>>, <<_asiprintf_r>>,
+        <<_siprintf_r>>, <<_sniprintf_r>>, <<_asniprintf_r>> are
+        simply reentrant versions of the functions above.
 
 RETURNS
-<<siprintf>> and <<asiprintf>> return the number of bytes in the output string,
-save that the concluding <<NULL>> is not counted.
-<<iprintf>> and <<fiprintf>> return the number of characters transmitted.
-If an error occurs, <<iprintf>> and <<fiprintf>> return <<EOF>> and
-<<asiprintf>> returns -1.  No error returns occur for <<siprintf>>.
+Similar to <<printf>>, <<fprintf>>, <<sprintf>>, <<snprintf>>, <<asprintf>>,
+and <<asnprintf>>.
 
 PORTABILITY
-<<iprintf>>, <<fiprintf>>, <<siprintf>>, <<sniprintf>>, and <<asprintf>>
-are newlib extensions.
+<<iprintf>>, <<fiprintf>>, <<siprintf>>, <<sniprintf>>, <<asiprintf>>,
+and <<asniprintf>> are newlib extensions.
 
 Supporting OS subroutines required: <<close>>, <<fstat>>, <<isatty>>,
 <<lseek>>, <<read>>, <<sbrk>>, <<write>>.
@@ -123,7 +129,7 @@ _siprintf_r(ptr, str, fmt, va_alist)
 #else
   va_start (ap);
 #endif
-  ret = _vfiprintf_r (ptr, &f, fmt, ap);
+  ret = _svfiprintf_r (ptr, &f, fmt, ap);
   va_end (ap);
   *f._p = 0;
   return (ret);
@@ -156,7 +162,7 @@ siprintf(str, fmt, va_alist)
 #else
   va_start (ap);
 #endif
-  ret = _vfiprintf_r (_REENT, &f, fmt, ap);
+  ret = _svfiprintf_r (_REENT, &f, fmt, ap);
   va_end (ap);
   *f._p = 0;
   return (ret);
