@@ -16,7 +16,7 @@ int
 _DEFUN(_vdprintf_r, (ptr, fd, format, ap),
        struct _reent *ptr _AND
        int fd _AND
-       const char *format _AND
+       const char *__restrict format _AND
        va_list ap)
 {
   char *p;
@@ -38,7 +38,7 @@ _DEFUN(_vdprintf_r, (ptr, fd, format, ap),
 int
 _DEFUN(vdprintf, (fd, format, ap),
        int fd _AND
-       const char *format _AND
+       const char *__restrict format _AND
        va_list ap)
 {
   return _vdprintf_r (_REENT, fd, format, ap);
