@@ -178,9 +178,17 @@ static char *rcsid = "$Id: vfprintf.c,v 1.43 2002/08/13 02:40:06 fitzsim Exp $";
 #endif
 
 #ifdef STRING_ONLY
-#define __SPRINT __ssprint_r
+# ifdef _FVWRITE_IN_STREAMIO
+#  define __SPRINT __ssprint_r
+# else
+#  define __SPRINT __ssputs_r
+# endif
 #else
-#define __SPRINT __sprint_r
+# ifdef _FVWRITE_IN_STREAMIO
+#  define __SPRINT __sprint_r
+# else
+#  define __SPRINT __sfputs_r
+# endif
 #endif
 
 /* The __sprint_r/__ssprint_r functions are shared between all versions of
@@ -188,6 +196,76 @@ static char *rcsid = "$Id: vfprintf.c,v 1.43 2002/08/13 02:40:06 fitzsim Exp $";
    the INTEGER_ONLY versions here. */
 #ifdef STRING_ONLY
 #ifdef INTEGER_ONLY
+#ifndef _FVWRITE_IN_STREAMIO
+int
+_DEFUN(__ssputs_r, (ptr, fp, buf, len),
+       struct _reent *ptr _AND
+       FILE *fp _AND
+       _CONST char *buf _AND
+       size_t len)
+{
+	register int w;
+
+	w = fp->_w;
+	if (len >= w && fp->_flags & (__SMBF | __SOPT)) {
+		/* must be asprintf family */
+		unsigned char *str;
+		int curpos = (fp->_p - fp->_bf._base);
+		/* Choose a geometric growth factor to avoid
+	 	 * quadratic realloc behavior, but use a rate less
+		 * than (1+sqrt(5))/2 to accomodate malloc
+	 	 * overhead. asprintf EXPECTS us to overallocate, so
+	 	 * that it can add a trailing \0 without
+	 	 * reallocating.  The new allocation should thus be
+	 	 * max(prev_size*1.5, curpos+len+1). */
+		int newsize = fp->_bf._size * 3 / 2;
+		if (newsize < curpos + len + 1)
+			newsize = curpos + len + 1;
+		if (fp->_flags & __SOPT)
+		{
+			/* asnprintf leaves original buffer alone.  */
+			str = (unsigned char *)_malloc_r (ptr, newsize);
+			if (!str)
+			{
+				ptr->_errno = ENOMEM;
+				goto err;
+			}
+			memcpy (str, fp->_bf._base, curpos);
+			fp->_flags = (fp->_flags & ~__SOPT) | __SMBF;
+		}
+		else
+		{
+			str = (unsigned char *)_realloc_r (ptr, fp->_bf._base,
+					newsize);
+			if (!str) {
+				/* Free unneeded buffer.  */
+				_free_r (ptr, fp->_bf._base);
+				/* Ensure correct errno, even if free
+				 * changed it.  */
+				ptr->_errno = ENOMEM;
+				goto err;
+			}
+		}
+		fp->_bf._base = str;
+		fp->_p = str + curpos;
+		fp->_bf._size = newsize;
+		w = len;
+		fp->_w = newsize - curpos;
+	}
+	if (len < w)
+		w = len;
+	(void)memmove ((_PTR) fp->_p, (_PTR) buf, (size_t) (w));
+	fp->_w -= w;
+	fp->_p += w;
+
+	return 0;
+
+err:
+	fp->_flags |= __SERR;
+	return EOF;
+}
+#endif
+
 int
 _DEFUN(__ssprint_r, (ptr, fp, uio),
        struct _reent *ptr _AND
@@ -280,11 +358,46 @@ err:
   return EOF;
 }
 #else /* !INTEGER_ONLY */
+#ifndef _FVWRITE_IN_STREAMIO
+int __ssputs_r (struct _reent *, FILE *, _CONST char *, size_t);
+#endif
 int __ssprint_r (struct _reent *, FILE *, register struct __suio *);
 #endif /* !INTEGER_ONLY */
 
 #else /* !STRING_ONLY */
 #ifdef INTEGER_ONLY
+
+#ifndef _FVWRITE_IN_STREAMIO
+int
+_DEFUN(__sfputs_r, (ptr, fp, buf, len),
+       struct _reent *ptr _AND
+       FILE *fp _AND
+       _CONST char *buf _AND
+       size_t len)
+{
+	register int i;
+
+#ifdef _WIDE_ORIENT
+	if (fp->_flags2 & __SWID) {
+		wchar_t *p;
+
+		p = (wchar_t *) buf;
+		for (i = 0; i < (len / sizeof (wchar_t)); i++) {
+			if (_fputwc_r (ptr, p[i], fp) == WEOF)
+				return -1;
+		}
+	} else {
+#else
+	{
+#endif
+		for (i = 0; i < len; i++) {
+			if (_fputc_r (ptr, buf[i], fp) == EOF)
+				return -1;
+		}
+	}
+	return (0);
+}
+#endif
 /*
  * Flush out all the vectors defined by the given uio,
  * then reset it so that it can be reused.
@@ -301,6 +414,7 @@ _DEFUN(__sprint_r, (ptr, fp, uio),
 		uio->uio_iovcnt = 0;
 		return (0);
 	}
+#ifdef _WIDE_ORIENT
 	if (fp->_flags2 & __SWID) {
 		struct __siov *iov;
 		wchar_t *p;
@@ -319,6 +433,7 @@ _DEFUN(__sprint_r, (ptr, fp, uio),
 			}
 		}
 	} else
+#endif
 		err = __sfvwrite_r(ptr, fp, uio);
 out:
 	uio->uio_resid = 0;
@@ -326,15 +441,21 @@ out:
 	return (err);
 }
 #else /* !INTEGER_ONLY */
+#ifndef _FVWRITE_IN_STREAMIO
+int __sfputs_r (struct _reent *, FILE *, _CONST char *buf, size_t);
+#endif
 int __sprint_r (struct _reent *, FILE *, register struct __suio *);
 #endif /* !INTEGER_ONLY */
 
+#ifdef _UNBUF_STREAM_OPT
 /*
  * Helper function for `fprintf to unbuffered unix file': creates a
  * temporary buffer.  We only work on write-only files; this avoids
  * worries about ungetc buffers and so forth.
+ *
+ * Make sure to avoid inlining.
  */
-static int
+_NOINLINE_STATIC int
 _DEFUN(__sbprintf, (rptr, fp, fmt, ap),
        struct _reent *rptr _AND
        register FILE *fp   _AND
@@ -372,11 +493,14 @@ _DEFUN(__sbprintf, (rptr, fp, fmt, ap),
 #endif
 	return (ret);
 }
+#endif /* _UNBUF_STREAM_OPT */
 #endif /* !STRING_ONLY */
 
 
-#ifdef FLOATING_POINT
+#if defined (FLOATING_POINT) || defined (_WANT_IO_C99_FORMATS)
 # include <locale.h>
+#endif
+#ifdef FLOATING_POINT
 # include <math.h>
 
 /* For %La, an exponent of 15 bits occupies the exponent character, a
@@ -423,8 +547,16 @@ static int exponent(char *, int, int);
    reentrant storage shared with mprec.  All other formats that use
    buf get by with fewer characters.  Making BUF slightly bigger
    reduces the need for malloc in %.*a and %S, when large precision or
-   long strings are processed.  */
+   long strings are processed.
+   The bigger size of 100 bytes is used on systems which allow number
+   strings using the locale's grouping character.  Since that's a multibyte
+   value, we should use a conservative value.
+   */
+#ifdef _WANT_IO_C99_FORMATS
+#define	BUF		100
+#else
 #define	BUF		40
+#endif
 #if defined _MB_CAPABLE && MB_LEN_MAX > BUF
 # undef BUF
 # define BUF MB_LEN_MAX
@@ -508,6 +640,9 @@ _EXFUN(get_arg, (struct _reent *data, int n, char *fmt,
 #else /* define as 0, to make SARG and UARG occupy fewer instructions  */
 # define CHARINT	0
 #endif
+#ifdef _WANT_IO_C99_FORMATS
+# define GROUPING	0x400		/* use grouping ("'" flag) */
+#endif
 
 int _EXFUN(_VFPRINTF_R, (struct _reent *, FILE *, _CONST char *, va_list));
 
@@ -535,7 +670,6 @@ _DEFUN(_VFPRINTF_R, (data, fp, fmt0, ap),
 	register int ch;	/* character from fmt */
 	register int n, m;	/* handy integers (short term usage) */
 	register char *cp;	/* handy char pointer (short term usage) */
-	register struct __siov *iovp;/* for PRINT macro */
 	register int flags;	/* flags as above */
 	char *fmt_anchor;       /* current format spec being processed */
 #ifndef _NO_POS_ARGS
@@ -552,6 +686,12 @@ _DEFUN(_VFPRINTF_R, (data, fp, fmt0, ap),
 	int width;		/* width from format (%8d), or 0 */
 	int prec;		/* precision from format (%.3d), or -1 */
 	char sign;		/* sign prefix (' ', '+', '-', or \0) */
+#ifdef _WANT_IO_C99_FORMATS
+				/* locale specific numeric grouping */
+	char *thousands_sep = NULL;
+	size_t thsnd_len = 0;
+	const char *grouping = NULL;
+#endif
 #ifdef FLOATING_POINT
 	char *decimal_point = _localeconv_r (data)->decimal_point;
 	size_t decp_len = strlen (decimal_point);
@@ -560,18 +700,28 @@ _DEFUN(_VFPRINTF_R, (data, fp, fmt0, ap),
 # define _fpvalue (_double_.fp)
 	int expt;		/* integer value of exponent */
 	int expsize = 0;	/* character count for expstr */
-	int ndig = 0;		/* actual number of digits returned by cvt */
 	char expstr[MAXEXPLEN];	/* buffer for exponent string */
+	int lead;		/* sig figs before decimal or group sep */
 #endif /* FLOATING_POINT */
+#if defined (FLOATING_POINT) || defined (_WANT_IO_C99_FORMATS)
+	int ndig = 0;		/* actual number of digits returned by cvt */
+#endif
+#if defined (FLOATING_POINT) && defined (_WANT_IO_C99_FORMATS)
+	int nseps;		/* number of group separators with ' */
+	int nrepeats;		/* number of repeats of the last group */
+#endif
 	u_quad_t _uquad;	/* integer arguments %[diouxX] */
 	enum { OCT, DEC, HEX } base;/* base for [diouxX] conversion */
 	int dprec;		/* a copy of prec if [diouxX], 0 otherwise */
 	int realsz;		/* field size expanded by dprec */
 	int size;		/* size of converted field or string */
 	char *xdigs = NULL;	/* digits for [xX] conversion */
+#ifdef _FVWRITE_IN_STREAMIO
 #define NIOV 8
 	struct __suio uio;	/* output information: summary */
 	struct __siov iov[NIOV];/* ... and individual io vectors */
+	register struct __siov *iovp;/* for PRINT macro */
+#endif
 	char buf[BUF];		/* space for %c, %S, %[diouxX], %[aA] */
 	char ox[2];		/* space for 0x hex-prefix */
 #ifdef _MB_CAPABLE
@@ -597,6 +747,7 @@ _DEFUN(_VFPRINTF_R, (data, fp, fmt0, ap),
 	/*
 	 * BEWARE, these `goto error' on error, and PAD uses `n'.
 	 */
+#ifdef _FVWRITE_IN_STREAMIO
 #define	PRINT(ptr, len) { \
 	iovp->iov_base = (ptr); \
 	iovp->iov_len = (len); \
@@ -617,12 +768,44 @@ _DEFUN(_VFPRINTF_R, (data, fp, fmt0, ap),
 		PRINT (with, n); \
 	} \
 }
+#define PRINTANDPAD(p, ep, len, with) { \
+	int n = (ep) - (p); \
+	if (n > (len)) \
+		n = (len); \
+	if (n > 0) \
+		PRINT((p), n); \
+	PAD((len) - (n > 0 ? n : 0), (with)); \
+}
 #define	FLUSH() { \
 	if (uio.uio_resid && __SPRINT(data, fp, &uio)) \
 		goto error; \
 	uio.uio_iovcnt = 0; \
 	iovp = iov; \
 }
+#else
+#define PRINT(ptr, len) {		\
+	if (__SPRINT (data, fp, (ptr), (len)) == EOF) \
+		goto error;		\
+}
+#define	PAD(howmany, with) {		\
+	if ((n = (howmany)) > 0) {	\
+		while (n > PADSIZE) {	\
+			PRINT (with, PADSIZE);	\
+			n -= PADSIZE;	\
+		}			\
+		PRINT (with, n);	\
+	}				\
+}
+#define PRINTANDPAD(p, ep, len, with) {	\
+	int n = (ep) - (p);		\
+	if (n > (len))			\
+		n = (len);		\
+	if (n > 0)			\
+		PRINT((p), n);		\
+	PAD((len) - (n > 0 ? n : 0), (with)); \
+}
+#define FLUSH()
+#endif
 
 	/* Macros to support positional arguments */
 #ifndef _NO_POS_ARGS
@@ -674,22 +857,24 @@ _DEFUN(_VFPRINTF_R, (data, fp, fmt0, ap),
 #ifndef STRING_ONLY
 	/* Initialize std streams if not dealing with sprintf family.  */
 	CHECK_INIT (data, fp);
-	_flockfile (fp);
+	_newlib_flockfile_start (fp);
 
 	ORIENT(fp, -1);
 
 	/* sorry, fprintf(read_only_file, "") returns EOF, not 0 */
 	if (cantwrite (data, fp)) {
-		_funlockfile (fp);
+		_newlib_flockfile_exit (fp);
 		return (EOF);
 	}
 
+#ifdef _UNBUF_STREAM_OPT
 	/* optimise fprintf(stderr) (and other unbuffered Unix files) */
 	if ((fp->_flags & (__SNBF|__SWR|__SRW)) == (__SNBF|__SWR) &&
 	    fp->_file >= 0) {
-		_funlockfile (fp);
+		_newlib_flockfile_exit (fp);
 		return (__sbprintf (data, fp, fmt0, ap));
 	}
+#endif
 #else /* STRING_ONLY */
         /* Create initial buffer if we are called by asprintf family.  */
         if (fp->_flags & __SMBF && !fp->_bf._base)
@@ -705,9 +890,11 @@ _DEFUN(_VFPRINTF_R, (data, fp, fmt0, ap),
 #endif /* STRING_ONLY */
 
 	fmt = (char *)fmt0;
+#ifdef _FVWRITE_IN_STREAMIO
 	uio.uio_iov = iovp = iov;
 	uio.uio_resid = 0;
 	uio.uio_iovcnt = 0;
+#endif
 	ret = 0;
 #ifndef _NO_POS_ARGS
 	arg_index = 0;
@@ -724,8 +911,13 @@ _DEFUN(_VFPRINTF_R, (data, fp, fmt0, ap),
 	        cp = fmt;
 #ifdef _MB_CAPABLE
 	        while ((n = __mbtowc (data, &wc, fmt, MB_CUR_MAX,
-				      __locale_charset (), &state)) > 0) {
-                    if (wc == '%')
+				      __locale_charset (), &state)) != 0) {
+		    if (n < 0) {
+			/* Wave invalid chars through. */
+			memset (&state, 0, sizeof state);
+			n = 1;
+		    }
+                    else if (wc == '%')
                         break;
                     fmt += n;
 		}
@@ -752,6 +944,12 @@ _DEFUN(_VFPRINTF_R, (data, fp, fmt0, ap),
 		width = 0;
 		prec = -1;
 		sign = '\0';
+#ifdef FLOATING_POINT
+		lead = 0;
+#ifdef _WANT_IO_C99_FORMATS
+		nseps = nrepeats = 0;
+#endif
+#endif
 #ifndef _NO_POS_ARGS
 		N = arg_index;
 		is_pos_arg = 0;
@@ -761,12 +959,12 @@ rflag:		ch = *fmt++;
 reswitch:	switch (ch) {
 #ifdef _WANT_IO_C99_FORMATS
 		case '\'':
-		  /* The ' flag is required by POSIX, but not C99.
-		     In the C locale, LC_NUMERIC requires
-		     thousands_sep to be the empty string.  And since
-		     no other locales are supported (yet), this flag
-		     is currently a no-op.  */
-		  goto rflag;
+			thousands_sep = _localeconv_r (data)->thousands_sep;
+			thsnd_len = strlen (thousands_sep);
+			grouping = _localeconv_r (data)->grouping;
+			if (thsnd_len > 0 && grouping && *grouping)
+			  flags |= GROUPING;
+			goto rflag;
 #endif
 		case ' ':
 			/*
@@ -1135,27 +1333,60 @@ reswitch:	switch (ch) {
 				size = expsize + ndig;
 				if (ndig > 1 || flags & ALT)
 					++size;
-			} else if (ch == 'f') {		/* f fmt */
-				if (expt > 0) {
+# ifdef _WANT_IO_C99_FORMATS
+				flags &= ~GROUPING;
+# endif
+			} else {
+				if (ch == 'f') {		/* f fmt */
+					if (expt > 0) {
+						size = expt;
+						if (prec || flags & ALT)
+							size += prec + 1;
+					} else	/* "0.X" */
+						size = (prec || flags & ALT)
+							  ? prec + 2
+							  : 1;
+				} else if (expt >= ndig) { /* fixed g fmt */
 					size = expt;
-					if (prec || flags & ALT)
-						size += prec + 1;
-				} else	/* "0.X" */
-					size = (prec || flags & ALT)
-						  ? prec + 2
-						  : 1;
-			} else if (expt >= ndig) {	/* fixed g fmt */
-				size = expt;
-				if (flags & ALT)
-					++size;
-			} else
-				size = ndig + (expt > 0 ?
-					1 : 2 - expt);
+					if (flags & ALT)
+						++size;
+				} else
+					size = ndig + (expt > 0 ?
+						1 : 2 - expt);
+# ifdef _WANT_IO_C99_FORMATS
+				if ((flags & GROUPING) && expt > 0) {
+					/* space for thousands' grouping */
+					nseps = nrepeats = 0;
+					lead = expt;
+					while (*grouping != CHAR_MAX) {
+						if (lead <= *grouping)
+							break;
+						lead -= *grouping;
+						if (grouping[1]) {
+							nseps++;
+							grouping++;
+						} else
+							nrepeats++;
+					}
+					size += (nseps + nrepeats) * thsnd_len;
+				} else
+# endif
+					lead = expt;
+			}
 
 			if (softsign)
 				sign = '-';
 			break;
 #endif /* FLOATING_POINT */
+#ifdef _GLIBC_EXTENSION
+		case 'm':  /* extension */
+			{
+				int dummy;
+				cp = _strerror_r (data, data->_errno, 1, &dummy);
+			}
+			flags &= ~LONGINT;
+			goto string;
+#endif
 		case 'n':
 #ifndef _NO_LONGLONG
 			if (flags & QUADINT)
@@ -1179,6 +1410,9 @@ reswitch:	switch (ch) {
 		case 'o':
 			_uquad = UARG ();
 			base = OCT;
+#ifdef _WANT_IO_C99_FORMATS
+			flags &= ~GROUPING;
+#endif
 			goto nosign;
 		case 'p':
 			/*
@@ -1200,8 +1434,11 @@ reswitch:	switch (ch) {
 #ifdef _WANT_IO_C99_FORMATS
 		case 'S':
 #endif
-			sign = '\0';
 			cp = GET_ARG (N, ap, char_ptr_t);
+#ifdef _GLIBC_EXTENSION
+string:
+#endif
+			sign = '\0';
 #ifndef __OPTIMIZE_SIZE__
 			/* Behavior is undefined if the user passed a
 			   NULL string when precision is not 0.
@@ -1315,6 +1552,9 @@ hex:			_uquad = UARG ();
 				flags |= HEXPREFIX;
 			}
 
+#ifdef _WANT_IO_C99_FORMATS
+			flags &= ~GROUPING;
+#endif
 			/* unsigned conversions */
 nosign:			sign = '\0';
 			/*
@@ -1350,11 +1590,37 @@ number:			if ((dprec = prec) >= 0)
 
 				case DEC:
 					/* many numbers are 1 digit */
-					while (_uquad >= 10) {
-						*--cp = to_char (_uquad % 10);
-						_uquad /= 10;
+					if (_uquad < 10) {
+						*--cp = to_char(_uquad);
+						break;
 					}
-					*--cp = to_char (_uquad);
+#ifdef _WANT_IO_C99_FORMATS
+					ndig = 0;
+#endif
+					do {
+					  *--cp = to_char (_uquad % 10);
+#ifdef _WANT_IO_C99_FORMATS
+					  ndig++;
+					  /* If (*grouping == CHAR_MAX) then no
+					     more grouping */
+					  if ((flags & GROUPING)
+					      && ndig == *grouping
+					      && *grouping != CHAR_MAX
+					      && _uquad > 9) {
+					    cp -= thsnd_len;
+					    strncpy (cp, thousands_sep,
+						     thsnd_len);
+					    ndig = 0;
+					    /* If (grouping[1] == '\0') then we
+					       have to use *grouping character
+					       (last grouping rule) for all
+					       next cases. */
+					    if (grouping[1] != '\0')
+					      grouping++;
+					  }
+#endif
+					  _uquad /= 10;
+					} while (_uquad != 0);
 					break;
 
 				case HEX:
@@ -1454,16 +1720,33 @@ number:			if ((dprec = prec) >= 0)
 						PAD (-expt, zeroes);
 						PRINT (cp, ndig);
 					}
-				} else if (expt >= ndig) {
-					PRINT (cp, ndig);
-					PAD (expt - ndig, zeroes);
-					if (flags & ALT)
-						PRINT (decimal_point, decp_len);
 				} else {
-					PRINT (cp, expt);
-					cp += expt;
-					PRINT (decimal_point, decp_len);
-					PRINT (cp, ndig - expt);
+					char *convbuf = cp;
+					PRINTANDPAD(cp, convbuf + ndig,
+						    lead, zeroes);
+					cp += lead;
+#ifdef _WANT_IO_C99_FORMATS
+					if (flags & GROUPING) {
+					    while (nseps > 0 || nrepeats > 0) {
+						if (nrepeats > 0)
+						    nrepeats--;
+						else {
+						    grouping--;
+						    nseps--;
+						}
+						PRINT(thousands_sep, thsnd_len);
+						PRINTANDPAD (cp, convbuf + ndig,
+							     *grouping, zeroes);
+						cp += *grouping;
+					    }
+					    if (cp > convbuf + ndig)
+						cp = convbuf + ndig;
+					}
+#endif
+					if (expt < ndig || flags & ALT)
+					    PRINT (decimal_point, decp_len);
+					PRINTANDPAD (cp, convbuf + ndig,
+						     ndig - expt, zeroes);
 				}
 			} else {	/* 'a', 'A', 'e', or 'E' */
 				if (ndig > 1 || flags & ALT) {
@@ -1503,7 +1786,7 @@ error:
 	if (malloc_buf != NULL)
 		_free_r (data, malloc_buf);
 #ifndef STRING_ONLY
-	_funlockfile (fp);
+	_newlib_flockfile_end (fp);
 #endif
 	return (__sferror (fp) ? EOF : ret);
 	/* NOTREACHED */
