@@ -24,30 +24,19 @@ static char sccsid[] = "%W% (Berkeley) %G%";
 #include <reent.h>
 #include <stdio.h>
 #include <limits.h>
-#ifdef _HAVE_STDC
 #include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
 
-#ifndef _REENT_ONLY 
+#include "local.h"
+
+#ifndef _REENT_ONLY
 
 int
 _DEFUN(vsprintf, (str, fmt, ap),
-       char *str        _AND
-       _CONST char *fmt _AND
+       char *__restrict str        _AND
+       const char *__restrict fmt _AND
        va_list ap)
 {
-  int ret;
-  FILE f;
-
-  f._flags = __SWR | __SSTR;
-  f._bf._base = f._p = (unsigned char *) str;
-  f._bf._size = f._w = INT_MAX;
-  f._file = -1;  /* No file. */
-  ret = _vfprintf_r (_REENT, &f, fmt, ap);
-  *f._p = 0;
-  return ret;
+  return _vsprintf_r (_REENT, str, fmt, ap);
 }
 
 #endif /* !_REENT_ONLY */
@@ -55,8 +44,8 @@ _DEFUN(vsprintf, (str, fmt, ap),
 int
 _DEFUN(_vsprintf_r, (ptr, str, fmt, ap),
        struct _reent *ptr _AND
-       char *str          _AND
-       _CONST char *fmt   _AND
+       char *__restrict str          _AND
+       const char *__restrict fmt   _AND
        va_list ap)
 {
   int ret;
@@ -66,8 +55,7 @@ _DEFUN(_vsprintf_r, (ptr, str, fmt, ap),
   f._bf._base = f._p = (unsigned char *) str;
   f._bf._size = f._w = INT_MAX;
   f._file = -1;  /* No file. */
-  ret = _vfprintf_r (ptr, &f, fmt, ap);
+  ret = _svfprintf_r (ptr, &f, fmt, ap);
   *f._p = 0;
   return ret;
 }
-
